package mschmidt.komo.datatools;
/*
 * Instances of this object class initialize, store, maintain, 
 * and make available variables used to connect to the database.  
 */

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.SQLException;
import java.util.Properties;
import mschmidt.komo.Application;
import mschmidt.komo.KomoPlugin;
import mschmidt.komo.preferences.PreferenceConstants;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.jobs.ILock;
import org.eclipse.jface.preference.IPreferenceStore;

public class ConnectionManager {

	/**
	 * Create the connection variables.
	 */

	private Driver dbDriver;

	private Connection dbConnection;

	private String returnMsg = new String();

	private IPreferenceStore store;

	/*
	 * Initialize a lock to be used by the Eclipse jobs API.  Database access
	 * is via a single connection but is done in worker threads.  This lock is 
	 * used to prevent simultaneous JDBC calls.
	 */
	public ILock dbLock = Platform.getJobManager().newLock();

	/**
	 * Create an instance of the ConnectionManager object 
	 * and initialize the variables. 
	 */
	public ConnectionManager() {
		store = KomoPlugin.getDefault().getPreferenceStore();
		dbDriver = null;
	}

	/**
	 * Method to provide the JDBC Driver needed to connect to the database.
	 * This is done using dynamic class loading technique.
	 * 
	 * @return drvInstance - the JDBC Driver (null if not available).
	 */
	private boolean setDriver() {

		// Get the JDBC jar file from the PreferenceStore.
		String jdbcURL = store.getString(PreferenceConstants.JDBC_PATH);
		File tempFile = new File(jdbcURL);
		boolean driverOK = false;

		if (dbDriver != null) {
			// If driver has been loaded, there is nothing to do except 
			// set the driverOK flag and the return message string.
			driverOK = true;
			returnMsg = "JDBC Driver " + dbDriver.getMajorVersion() + "."
					+ dbDriver.getMinorVersion() + ".";
		} else if (jdbcURL.length() == 0 || jdbcURL == null) {
			// Check if the user has entered the Driver path.
			if (dbDriver != null) {
				dbDriver = null;
			}
			returnMsg = "ERROR: JDBC has not been set.";
		} else if (!tempFile.isFile()) {
			// Check if the user-entered path is a file.  
			if (dbDriver != null) {
				dbDriver = null;
			}
			returnMsg = "ERROR: " + jdbcURL + " is not a valid file.";
		} else {
			// Load the driver.  This is done once per session.
			try {
				URL[] url = new URL[1];
				url[0] = new File(jdbcURL).toURL();
				URLClassLoader loader = new URLClassLoader(url);
				Class drvClass = loader.loadClass("org.postgresql.Driver");
				dbDriver = (Driver) drvClass.newInstance();
				driverOK = true;
				returnMsg = "JDBC Driver " + dbDriver.getMajorVersion() + "."
						+ dbDriver.getMinorVersion() + ".";
			} catch (Exception e) {
				returnMsg = "ERROR: Unable to load JDBC.";
				if (dbDriver != null) {
					dbDriver = null;
					System.out.println(e.getMessage());
					e.printStackTrace();
				}
			}
		}
		// return the Driver for use in making a connection.
		return driverOK;
	}

	/**
	 * Method to set and return the database connection.
	 * 
	 * @see closeConnection - used to close the connection once
	 * processing is completed
	 * 
	 * @return dbConnection - the database connection, 
	 * null if unavailable.
	 * @throws SQLException 
	 */
	public final Connection getConnection() {

		// Test if there is an open connection.
		boolean noConnection = false;
		if (dbConnection == null) {
			noConnection = true;
		} else {
			try {
				noConnection = dbConnection.isClosed();
			} catch (SQLException e) {
				noConnection = true;
				if (dbConnection != null) {
					dbConnection = null;
				}
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}
		// if the connection is not available, create it.
		if (noConnection) {
			// Test for a valid driver.
			if (setDriver()) {
				// Get password and test if it is valid.
				String thisPassword = Application.dbPassword.getPassword();
				if (thisPassword != null && thisPassword.length() > 0) {
					// Load connection values from the PreferenceStore
					String dbName = store
						.getString(PreferenceConstants.DB_NAME);
					String dbUser = store
						.getString(PreferenceConstants.DB_USER);
					String dbPort = store
						.getString(PreferenceConstants.DB_PORT);
					String dbURL = store.getString(PreferenceConstants.DB_URL);
					boolean dbSSL = store
						.getBoolean(PreferenceConstants.DB_SSL);
					// Test for valid connection values.
					if (dbURL != null && dbURL.length() > 0 && dbPort != null
						&& dbPort.length() > 0 && dbName != null
						&& dbName.length() > 0) {

						String dbConnectString = "jdbc:postgresql://" + dbURL
								+ ":" + dbPort + "/" + dbName;
						// Set connection properties.
						Properties connectProp = new Properties();
						connectProp.put("user", dbUser);
						connectProp.put("password", thisPassword);
						// Additional properties for basic 
						// Secure Socket Layer connection.
						if (dbSSL) {
							connectProp.put("ssl", "true");
							connectProp.put("sslfactory",
									"org.postgresql.ssl.NonValidatingFactory");
							returnMsg = returnMsg + " SSL on.";
						} else {
							returnMsg = returnMsg + " SSL off.";
						}
						// Standard JDBC connection creation statements. 
						try {
							dbConnection = dbDriver.connect(dbConnectString,
									connectProp);
						} catch (SQLException e) {
							if (dbConnection != null) {
								dbConnection = null;
							}
							String s = e.getMessage();
							if (s.startsWith("FATAL: ")) {
								s = s.substring(8);
							}
							returnMsg = "ERROR: " + s;
							System.out.println(e.getMessage());
							e.printStackTrace();
						}
					} else {
						returnMsg = "ERROR: Invalid settings.";
						if (dbConnection != null) {
							dbConnection = null;
						}
					}
				} else {
					returnMsg = "ERROR: Password not entered";
					if (dbConnection != null) {
						dbConnection = null;
					}
				}
			} else {
				if (dbConnection != null) {
					dbConnection = null;
				}
			}
		}

		return dbConnection;
	}

	/**
	 * Method to close the database connection.
	 */
	public final void closeConnection() {
		if (dbConnection != null) {
			try {
				if (!dbConnection.isClosed()) {
					dbConnection.close();
				}
			} catch (SQLException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

	}

	/**
	 * Return driver status information.
	 * This is typically shown in the status line 
	 * at the bottom of the application window.  
	 * 
	 * @return returnMsg - current status message
	 */
	public final String getMessage() {
		return returnMsg;
	}

	public final Driver getDriver() {
		return dbDriver;
	}

}
